/*----------------------------------------------------------libint.c----------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                 and                                                        */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

/* 
 *	Common read/write routines for sockets
 */

#include	"../common/inc.h"
#include	"../common/protodefs.h"

#define EXTERN extern

/*------------------------------------------------_WaitForReadable()----------*/
/*
 *	_WaitForReadable -- wait on the connection till enough bytes come
 */

void	_WaitForReadable(conn)
FD	conn;
{
	fd_set	r_mask;
	int		result;

	FD_ZERO(&r_mask);	/* Clear mask */

	do{
		FD_SET(conn, &r_mask);
		result = select(conn +1, &r_mask, NULL, NULL, NULL);

		if(result == -1 && errno != EINTR )
		{
			perror("Error in select");
			exit(1);
		}

	 }while( result <= 0);

}

/*------------------------------------------------------------Read()----------*/
/*
 *	_Read - Read bytes from the connection taking into account incomplete
 *	reads. 
 */

Status	_Read(conn, data, size)
register FD	conn;
register char 	*data;
register long	size;
{
	register long	bytes_read;

	if(size == 0) return(FAILURE);
	errno= 0;

	while((bytes_read = read(conn, data, (int)size)) != size)
	{
		if(bytes_read > 0)
		{
			size -= bytes_read;
			data += bytes_read;
		}

#ifdef 	EWOULDBLOCK
		else if(errno == EWOULDBLOCK )
			{
				_WaitForReadable(conn);
				errno = 0;
			}
#endif	/* EWOULDBLOCK */

		else if(bytes_read == 0)	/* read failed due to EOF */
			{
				errno = EPIPE;
				ERROR("Read failed due to EOF");
				return(FAILURE);
			}
		else	/* read retd < 0 ; presumably -1 */
		{
			if(errno != EINTR)
				perror("Read failed");

			return(FAILURE);

		}
	}	/* End of while */

	return(SUCCESS);

}

/*----------------------------------------------------------_Write()----------*/
/* 
 *	_Write -- Analogous to _Read. Puts bytes on the connection
 */

Status	_Write(conn, size, buf)
register FD	conn;
register long	size;
register char 	*buf;
{
	int		n;
	long	bytes_left = size;

	unsigned char *p = (unsigned char *)buf;

	/*ENTER_PROC("_Write");*/

	while(bytes_left > 0)
	{
		if(( n = write(conn, (char *)p, (int)bytes_left)) > 0)
		{
			bytes_left -= n;
			p += n;
		}
		else
		{
			SYSFATAL("Error in write");
			return(FAILURE);
		}

	}	/* end of while */

	return(SUCCESS);

}

/*-----------------------------------------------------WriteToClient()--------*/
/*
 *	WriteToClient-- Currently simply puts bytes on the connection
 *	To be reworked later to provide buffering 
 */

Status	WriteToClient(fd, n, buf)
FD		fd;
unsigned int n;
char *	buf;
{
	Status  st;

	st = _Write(fd, n, buf);

	return(st);
}

/*-----------------------------------------------------WriteToServer()--------*/
/*
 *	WriteToServer-- Currently simply puts bytes on the connection
 *	To be reworked later to provide buffering 
 */

Status	WriteToServer(fd, n, buf)
FD	fd;
unsigned int n;
char *buf;
{
	Status  st;

	st = _Write(fd, n, buf);

	return(st);
}

/*--------------------------------------------------------ReadBuffer()--------*/
/*
 *	ReadBuffer-- Currently simply reads bytes from the connection.
 *	To be reworked later to provide buffering 
 */

Status	ReadBuffer(fd, buf, n)
FD	fd;
char *buf;
unsigned int n;
{
	Status  st;

	st = _Read(fd, buf, n);

	return(st);

}

/*------------------------------------------------------ProcessError()--------*/
/*
 *	ProcessError  -- Processes any T_Error packets
 */

Status	ProcessError(terr)
T_Error	*terr; 
{
	/* Now print the error fields */
	
	/*WORKHERE*/	 
	return(SUCCESS);
}

/*------------------------------------------------------------_Reply()--------*/
/*
 *	_Reply -- Waits for the correct reply packet on the connection
 *	While waiting it may encounter the foll:
 *
 *	1.	T_Error: call ProcessError routine.
 *	2.	T_Data:  Enqueue the data in the correct group Q
 *	3.	T_Reply: If this is not the T_Reply we need then panic !!
 */

Status	_Reply(fd, replycode, rep)
FD	fd;
int	replycode;
T_GenericReply *rep;
{
	T_GenericReply  trep;
	Status			st;

	while(1)
	{
		if(!ReadBuffer(fd, (char *)&trep, sizeof(T_GenericReply)))
		{
			ERROR("Error in ReadBuffer");
			return(FAILURE);
	 	}

		switch(trep.type)
	 	{

		case _T_Error:
				{
				T_Error	terr;

				bcopy((char *)&trep, (char *)&terr, 
						sizeof(T_GenericReply));
				if((st =ProcessError(&terr)) != SUCCESS)
				{
					ERROR("ProcessError failed");
					return(st);
				}

				break;
				}
		case _T_Data :
				{
				T_Data tdata;

				bcopy((char *)&trep, (char *)&tdata, 
							sizeof(T_GenericReply));

				break;
				}

		case _T_Reply:
				{
				if((trep.opcode )!= replycode)
				{
					ERROR( "Read wrong reply from connection");
					return(FAILURE);
				}

				/* else */
				switch(replycode)
				{

					case _MakeGroupReply:
				    	{
					    MakeGroupReply reply;

					    bcopy((char *)&trep,
						  (char *)&reply,
						  sizeof(MakeGroupReply));
					    bcopy((char *)&reply,
						  (char *)rep,
						  sizeof(MakeGroupReply));
				
					    return(SUCCESS);
					    break;
					}
					case _SendReply:
				    	{
					    SendReply reply;

					    bcopy((char *)&trep, (char *)&reply, sizeof(SendReply));
					    bcopy((char *)&reply, (char *)rep, sizeof(SendReply));
				
					    return(SUCCESS);
					    break;
					}

					case _AudioStartReply:
					{
					    AudioStartReply reply;

					    bcopy((char *)&trep,
						  (char *)&reply,
						  sizeof(AudioStartReply));
					    bcopy((char *)&reply,
						  (char *)rep,
						  sizeof(AudioStartReply));
				
					    return(SUCCESS);
					    break;
					}

					case _ImageStartReply:
					{
					    ImageStartReply reply;

					    bcopy((char *)&trep, (char *)&reply,
						  sizeof(AudioStartReply));
					    bcopy((char *)&reply, (char *)rep,
						  sizeof(AudioStartReply));
				
					    return(SUCCESS);
					    break;
					}


				 	case _HistoryReqReply:
					{
						HistoryReqReply reply;

						bcopy((char *)&trep,
							(char *)&reply,
						sizeof(HistoryReqReply));

						bcopy((char *)&reply,
							(char *)rep,
						sizeof(HistoryReqReply));
				
						return(SUCCESS);
						break;
					}

				 	case _JoinGroupReply:
					{
						JoinGroupReply reply;

						bcopy((char *)&trep,
							(char *)&reply,
						sizeof(JoinGroupReply));

						bcopy((char *)&reply,
							(char *)rep,
							sizeof(JoinGroupReply));
				
						return(SUCCESS);
						break;
					}

				 	case _LeaveGroupReply:
					{
						LeaveGroupReply reply;

						bcopy((char *)&trep,
							(char *)&reply,
						sizeof(LeaveGroupReply));

						bcopy((char *)&reply,
							(char *)rep,
						sizeof(LeaveGroupReply));
				
						return(SUCCESS);
						break;
					}

				default:
						ERROR("#Wrong replycode");
						return(FAILURE);
						break;

				}	/* end of inner switch */

			   break;

			   }		/* End of case T_Reply */

		default:
			ERROR("$Wrong replycode ");
			return(FAILURE);
			break;

	}		/* End of outer switch */

    }		/* End of while */

}

/*--------------------------------------------------End of libint.c ----------*/
