/*-----------------------------------------------------localrout.c------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                    and                                                     */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

#include	"../common/protodefs.h"
#include	"localrout.h"
#include 	"ex_glbls.h"
#include 	"ex_routines.h"
#include 	"fncproto.h"

/*-------------------------------------------------------mk_new_user()--------*/
/* This routine stores the participants information in the array
 * Returns error if maxparticipants reached
 */
/*----------------------------------------------------------------------------*/
Status mk_new_user(sock,name, photodata, psize)	
FD	sock;
char	*name;
char	*photodata;
int		psize;
{
	int	i;
	FILE	*fp;

	ENTER_PROC("mk_new_user");

	for (i = 0; i < MAXPARTICIPANTS; i++ ) 
		if (pinfo[i].used == FALSE) 
		{
			pinfo[i].used = TRUE;
			pinfo[i].fd	= sock ;
			pinfo[i].Name = (char *)Malloc( strlen(name)*sizeof(char)) ;
			if ( pinfo[i].Name != NULL )
				strcpy(pinfo[i].Name,name);
			pinfo[i].Description = (char *)Malloc(20) ; /*WORKHERE*/
			strcpy(pinfo[i].Description,"NONE") ;
			/* photofile */
			/* generate a temporary name */
			pinfo[i].pfname = tempnam(NULL, "VPH") ;
			/* write photodata into that file */
			if ( (fp = fopen(pinfo[i].pfname,"w") ) == NULL)
			{
				/* Do error processing */
				FATAL("Cannot open temporary file ") ;
				
			}
			for (i=0 ; i<psize; i++)
				fputc( *photodata++, fp) ;

			fclose(fp) ;
	
			return(SUCCESS);
		}

DEBUG_LEVEL2((stderr,"MADE NEW USER \n")) ;

	return (_NoSpaceLeft);
}


/*-----------------------------------------------------init_group_tab()-------*/
/* Initialise the grouptable and the participants info. table
 * to unused.
 */
/*----------------------------------------------------------------------------*/
void	init_group_tab ()
{

	int	i, j;

	ENTER_PROC("init_group_tab") ;

	for (i=0; i< VNGROUPS; i++)
	{
		grouptable[i]=(GRP_TAB *)NULL ;
		participantsnumber[i] = 0 ;
	}

	for ( i=0 ; i<MAXPARTICIPANTS; i++)
	{
		pinfo[i].used = FALSE ;
		pinfo[i].pfname = NULL ;
	}

	for (i=0; i<VNGROUPS; i++)
	{
		image_channel_status[i] = FREE ;
		audio_channel_status[i] = FREE ;
	}

}

/*----------------------------------------------------------UserName()--------*/
/* Given the socket fd, this gets the name of the
 * participants connected to it.
 */
/*----------------------------------------------------------------------------*/
Status UserName(sock,name)	/* given sock get the username */
FD	sock;
char	**name;
{

	PARTICIPANTS_INFO	*pinfo ;
	ENTER_PROC("UserName");

	pinfo = getpinfo( sock) ;
	if (pinfo == NULL)
		return(FAILURE) ;
	
	*name = (char*) Malloc((unsigned int) (strlen(pinfo->Name)));
	strcpy(*name,pinfo->Name);
	return(SUCCESS);
}

/*----------------------------------------------------DistribAuSocks()--------*/
/* This routine finds out all the fds which are entitled to
 * receive audio packets send by client(fromsock) in group (id)
 */
/*----------------------------------------------------------------------------*/

Status DistribAuSocks(id,array,nelemp,fromsock)  
int	id;
FD	**array;
int	*nelemp;
FD	fromsock;	/* came from where so do not put this in array */
{
	int count = 0;
	int arrind = 0;
	int j;

	ENTER_PROC("DistribAuSocks");
	if ((id<0) || (id>VNGROUPS))
		return(FAILURE);

	if (participantsnumber[id] == 0)
		return(SUCCESS);  /*WORKHERE*/ 

	(*array) = (FD *) Malloc((participantsnumber[id])*sizeof(FD));

	for (j = 0; j< participantsnumber[id]; j++) 
	{
		if ( groupaufds[id][j] != fromsock)
		{
			(*array)[arrind++] = groupaufds[id][j] ;
		}
	}

	*nelemp = arrind ;

	return(SUCCESS);
}

/*----------------------------------------------------DistribImgSocks()--------*/
/* This routine finds out all the fds which are entitled to
 * receive image packets send by client(fromsock) in group (id)
 */
/*----------------------------------------------------------------------------*/

Status DistribImgSocks(id,array,nelemp,fromsock)  
int	id;
FD	**array;
int	*nelemp;
FD	fromsock;	/* came from where so do not put this in array */
{
	int count = 0;
	int arrind = 0;
	int j;

	ENTER_PROC("DistribImgSocks");
	if ((id<0) || (id>VNGROUPS))
		return(FAILURE);

	if (participantsnumber[id] == 0)
		return(SUCCESS);  /*WORKHERE*/ 

	(*array) = (FD *) Malloc((participantsnumber[id])*sizeof(FD));

	for (j = 0; j< participantsnumber[id]; j++) 
		if ( groupimgfds[id][j] != fromsock)
			(*array)[arrind++] = groupimgfds[id][j] ;

	*nelemp = arrind ;

	return(SUCCESS);
}

/*------------------------------------------------------DistribSocks()--------*/
/*------------------------------------------------------DistribSocks()--------*/
/* This routine finds out all the fds which are entitled to
 * receive text packets send by client(fromsock) in group (id)
 */
/*----------------------------------------------------------------------------*/
Status DistribSocks(id,array,nelemp,fromsock)  
int	id;
FD	**array;
int	*nelemp;
FD	fromsock;	/* came from where so do not put this in array */
{
	int count = 0;
	int arrind = 0;
	int j;

	ENTER_PROC("DistribSocks");
	if ((id<0) || (id>VNGROUPS))
		return(FAILURE);


	if (participantsnumber[id] == 0)
		return(SUCCESS);  /*WORKHERE*/ 

	(*array) = (FD *) Malloc((participantsnumber[id])*sizeof(FD));

	for (j = 0; j< participantsnumber[id]; j++) 
	{
		if ( groupfds[id][j] != fromsock)
			(*array)[arrind++] = groupfds[id][j] ;
	}

	*nelemp = arrind ;

	return(SUCCESS);
}

/*-----------------------------------------------------clean_rm_proc()--------*/
/* Removes the user from grouptable. 
 * Also remove the participants info entry
 */
/*----------------------------------------------------------------------------*/
Status clean_rm_proc(sock)	
FD sock;
{
	int i,j;

	ENTER_PROC("clean_rm_proc");

	/* Remove the users fd from all the groups joined and also remove
		the users info */
	for ( i=0; i<VNGROUPS; i++)
	{
		removefromgroupfds( sock, i) ;
		/*deleteuser( sock, i) ;*//*WORKHERE*/  

	/* Freeing the memory allocated to store user info */
		/*WORKHERE*/ 
	}

	return(SUCCESS) ;
	
}

/*-------------------------------------------------End of localrout.c---------*/
