/*------------------------------------------------------waitfor.c-------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                     and                                                    */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/


/*
 *	This file stores the WaitForSomething routine that waits for input
 *	and then reports it to the server
 */

#include	"../common/inc.h"
#include	"ex_glbls.h"
#include	"ex_routines.h"
#include	"fncproto.h"


/*-----------------------------------------------------WaitForSomething()-----*/
/* Do a select on all the clients that are connected.
 * On +ve return from select, return the r_mask indicating
 * the ports ready to be read
 */ 
/*----------------------------------------------------------------------------*/
int	WaitForSomething( r_mask )
fd_set	*r_mask;		/* return */
{
	fd_set	rfds, wfds, xfds;
	short	nfds;
	int	no_rfds = 0;		/* Return value */
	FD	fd;

	ENTER_PROC("WaitForSomething");

	FD_ZERO(r_mask);	/* First clear the return mask */

	do{

		rfds = ReadDescriptors;	/* global mask of all curr FDs*/
		xfds = rfds;
	FD_ZERO(&wfds);

	nfds	= 	select(HighestFD + 1, &rfds, &wfds, &xfds, NULL);

	/* Now process what select retd */

	if(nfds < 0)
	{
		if(errno == EINTR )
			continue ;	/* To end of while loop */
		if(errno == EBADFD)
		{
			/* One of the bits in rfds is invalid . Close down files
			   until it goes away */
			RemoveFD(HighestFD);
			continue ;	/* To end of while loop */
		}
	
		perror("error in select");
		continue;

	}

	if(nfds == 0)
	{
		ERROR("select retd nfds: == 0");
		perror("select");
		continue;

	}

	/* Check each fd for input and return mask of all that do */

	for( fd = 0; 0 < nfds && fd <= HighestFD ; fd++)
	{
		if(!FD_ISSET(fd, &rfds))
			continue ;

		if(FD_ISSET(fd, &xfds))
		{
			DEBUG_LEVEL1((stderr,"XCEPTION COND ON FD:%d\n",fd));
			
			if(FD_ISSET(fd,&ReadDescriptors))
		 		CloseDownClient(fd);

		 	continue;
		}
		/* else */

		DEBUG_LEVEL1((stderr,"FD %d SET\n",fd));

		nfds-- ;
		no_rfds++;	/* to be retd */

		FD_SET(fd, r_mask );

	 }

     }while( no_rfds <= 0);

	return( no_rfds );

}

/*-------------------------------------------------------------MainLoop()-----*/
/*
 *	MainLoop -- The main dispatch loop of the server . waits till something
 * 	is retd by WaitForSomething() and then processes
 */
/*----------------------------------------------------------------------------*/

void	MainLoop()
{
int	nready;
FD	fd;

extern	FD	AcceptClientConnection();

	/* Infinite dispatch loop */

	while(TRUE)
	{
		fd_set	r_mask;

		FD_ZERO(&r_mask);	/* Clear r_mask */

		if((nready = WaitForSomething( &r_mask )) <=  0 )
			continue;	/* To end of while loop */

		 /* else */

		 DEBUG_LEVEL3((stderr,"nready retd by WaitForSomethIng:%d\n",
				nready));

		 {
		 long	pid;
		 char	username[MAXNAMESZ];
		 char	*photodata ;

		int		psize ;
			/* First check for any new connections */


			if(FD_ISSET(InputPortFD, &r_mask))
			{

				DEBUG_LEVEL2((stderr,"FOUND AT LEAST ONE NEW CLIENT"));

				if((fd = AcceptClientConnection(InputPortFD, 
						username, &photodata, &psize, (int *)&pid)) < 0)
				{
					ERROR("Client FD retd was negative");
					/*continue ;*/
				}
				else
				{

				AddFD(fd);

				if((mk_new_user(fd, username, photodata, psize)) != SUCCESS)
				{
					RemoveFD(fd);
					ERROR("Couldnt add client to list");
				}

				/* Clear the username array */
			/*WORKHERE*/	

				username[0] = '\0';

			   	FD_CLR(fd, &r_mask);
			   	if((--nready) <= 0)
			    	{
					DEBUG_LEVEL3((stderr,"nready now ZERO"));
					continue;	/* To start of while loop */
			    	}

			    }	

			 }		/* End of if for InputPortFD */
	


		  }			/* End of check for new connections */

		  /* Now process other connections */
		  /* Currently ignoring exception conditions - add later */

		  fd = 0;

		  for( fd = 0 ; fd <= HighestFD; fd++)
		   {

			if(!FD_ISSET(fd, &r_mask))
				continue;

			/* As we've already checked for new clients 
			   Now Process requests from other clients */

			if(FD_ISSET(fd,&r_mask))
			 {
				DEBUG_LEVEL2((stderr,"FOUND INPUT ON FD:%d\n",fd));
				ProcessReqFromClient(fd);

				FD_CLR(fd, &r_mask);
			   	if((--nready) <= 0)
			    	{
					DEBUG_LEVEL3((stderr,"nready now ZERO"));
					break;
			    	}
			 }

		    }			/* End of for for clients*/

		}			/* End of while */

}

/*-------------------------------------------------------End of waitfor.c-----*/

